package jdk.internal.cmm;

import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;
import javax.management.AttributeChangeNotification;
import javax.management.MBeanNotificationInfo;
import java.lang.management.ManagementPermission;
import jdk.management.cmm.SystemResourcePressureMXBean;
import javax.management.NotificationBroadcasterSupport;

public final class SystemResourcePressureImpl extends NotificationBroadcasterSupport implements SystemResourcePressureMXBean
{
    private static final int MIN_PRESSURE_LEVEL = 0;
    private static final int MAX_PRESSURE_LEVEL = 10;
    public static final String RESOURCE_PRESSURE_MXBEAN_NAME = "com.oracle.management:type=ResourcePressureMBean";
    private static final String MEM_PRESSURE_ATTRIBUTE_NAME = "MemoryPressure";
    private long notifSeqNum;
    private static ManagementPermission controlPermission;
    
    public SystemResourcePressureImpl() {
        MBeanNotificationInfo[] info = { null };
        info[0] = new MBeanNotificationInfo(new String[] { "jmx.attribute.change" }, AttributeChangeNotification.class.getName(), "Notification that Memory pressure level has changed");
        super(info);
    }
    
    public synchronized int getMemoryPressure() {
        return this.getVmMemoryPressure();
    }
    
    public void setMemoryPressure(int i) {
        if (i < 0 || i > 10) {
            throw new IllegalArgumentException("Invalid pressure level: " + i);
        }
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPermission(SystemResourcePressureImpl.controlPermission);
        }
        AttributeChangeNotification notification;
        synchronized (this) {
            int setVmMemoryPressure = this.setVmMemoryPressure(i);
            if (i == setVmMemoryPressure) {
                return;
            }
            long n = this.notifSeqNum + 1L;
            this.notifSeqNum = n;
            notification = new AttributeChangeNotification(this, n, System.currentTimeMillis(), "Memory pressure level change detected", "MemoryPressure", "int", Integer.valueOf(setVmMemoryPressure), Integer.valueOf(i));
        }
        this.sendNotification(notification);
        return;
    }
    
    private native int setVmMemoryPressure(int p0);
    
    private native int getVmMemoryPressure();
    
    public ObjectName getObjectName() {
        try {
            return ObjectName.getInstance("com.oracle.management:type=ResourcePressureMBean");
        }
        catch (MalformedObjectNameException ex) {
            throw new InternalError();
        }
    }
    
    static {
        SystemResourcePressureImpl.controlPermission = new ManagementPermission("control");
    }
}
